#ifndef CYLSIN_EVAL_T_H
#define CYLSIN_EVAL_T_H

#include <cmath>
#include <vector>
#include "tensorCompute/trig_integral.h"

#define SinT2Pi TrigInt::IntegrateSin2Pi_D2
#define CosT2Pi TrigInt::IntegrateCos2Pi_D2
#define Pi M_PI


class cylSintEval{
public:
cylSintEval();
std::vector<double (*)(const int, const int, const int)> pointers_;
~cylSintEval(){};
/* Sin[h2*t]Sin[i2*t]Cos[g2*t]*/
static double evalT_0(const int i2x2, const int g2x2, const int h2x2){
return (-CosT2Pi(g2x2-h2x2-i2x2)+CosT2Pi(g2x2+h2x2-i2x2)+CosT2Pi(g2x2-h2x2+i2x2)-CosT2Pi(g2x2+h2x2+i2x2))/4.;
};

/* Sin[g2*t]Sin[i2*t]Cos[h2*t]*/
static double evalT_1(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(g2x2-h2x2-i2x2)+CosT2Pi(g2x2+h2x2-i2x2)-CosT2Pi(g2x2-h2x2+i2x2)-CosT2Pi(g2x2+h2x2+i2x2))/4.;
};

/* Cos[g2*t]Cos[h2*t]Cos[i2*t]*/
static double evalT_2(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(g2x2-h2x2-i2x2)+CosT2Pi(g2x2+h2x2-i2x2)+CosT2Pi(g2x2-h2x2+i2x2)+CosT2Pi(g2x2+h2x2+i2x2))/4.;
};

/* Sin[g2*t]Sin[h2*t]Sin[i2*t]*/
static double evalT_3(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(g2x2-h2x2-i2x2)+SinT2Pi(g2x2+h2x2-i2x2)+SinT2Pi(g2x2-h2x2+i2x2)-SinT2Pi(g2x2+h2x2+i2x2))/4.;
};

/* Sin[i2*t]Cos[g2*t]Cos[h2*t]*/
static double evalT_4(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(g2x2-h2x2-i2x2)-SinT2Pi(g2x2+h2x2-i2x2)+SinT2Pi(g2x2-h2x2+i2x2)+SinT2Pi(g2x2+h2x2+i2x2))/4.;
};

/* Sin[h2*t]Cos[g2*t]Cos[i2*t]*/
static double evalT_5(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(g2x2-h2x2-i2x2)+SinT2Pi(g2x2+h2x2-i2x2)-SinT2Pi(g2x2-h2x2+i2x2)+SinT2Pi(g2x2+h2x2+i2x2))/4.;
};

/* Sin[g2*t]Sin[i2*t]Cos[2*t]*/
static double evalT_6(const int i2x2, const int g2x2, const int h2x2){
return (-CosT2Pi(2-g2x2-i2x2)+CosT2Pi(2+g2x2-i2x2)+CosT2Pi(2-g2x2+i2x2)-CosT2Pi(2+g2x2+i2x2))/4.;
};

/* Sin[2*t]Sin[i2*t]Cos[g2*t]*/
static double evalT_7(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-g2x2-i2x2)+CosT2Pi(2+g2x2-i2x2)-CosT2Pi(2-g2x2+i2x2)-CosT2Pi(2+g2x2+i2x2))/4.;
};

/* Cos[2*t]Cos[g2*t]Cos[i2*t]*/
static double evalT_8(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-g2x2-i2x2)+CosT2Pi(2+g2x2-i2x2)+CosT2Pi(2-g2x2+i2x2)+CosT2Pi(2+g2x2+i2x2))/4.;
};

/* Sin[2*t]Sin[g2*t]Sin[i2*t]*/
static double evalT_9(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-g2x2-i2x2)+SinT2Pi(2+g2x2-i2x2)+SinT2Pi(2-g2x2+i2x2)-SinT2Pi(2+g2x2+i2x2))/4.;
};

/* Sin[i2*t]Cos[2*t]Cos[g2*t]*/
static double evalT_10(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-g2x2-i2x2)-SinT2Pi(2+g2x2-i2x2)+SinT2Pi(2-g2x2+i2x2)+SinT2Pi(2+g2x2+i2x2))/4.;
};

/* Sin[2*t]Cos[g2*t]Cos[i2*t]*/
static double evalT_11(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(2-g2x2-i2x2)+SinT2Pi(2+g2x2-i2x2)+SinT2Pi(2-g2x2+i2x2)+SinT2Pi(2+g2x2+i2x2))/4.;
};

/* Sin[g2*t]Sin[i2*t]*/
static double evalT_12(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(g2x2-i2x2)-CosT2Pi(g2x2+i2x2))/2.;
};

/* Cos[g2*t]Cos[i2*t]*/
static double evalT_13(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(g2x2-i2x2)+CosT2Pi(g2x2+i2x2))/2.;
};

/* Sin[i2*t]Cos[g2*t]*/
static double evalT_14(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(g2x2-i2x2)+SinT2Pi(g2x2+i2x2))/2.;
};

/* Sin[g2*t]Cos[h2*t]Cos[i2*t]*/
static double evalT_15(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(g2x2-h2x2-i2x2)+SinT2Pi(g2x2+h2x2-i2x2)+SinT2Pi(g2x2-h2x2+i2x2)+SinT2Pi(g2x2+h2x2+i2x2))/4.;
};

/* Sin[g2*t]Sin[h2*t]Cos[i2*t]*/
static double evalT_16(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(g2x2-h2x2-i2x2)-CosT2Pi(g2x2+h2x2-i2x2)+CosT2Pi(g2x2-h2x2+i2x2)-CosT2Pi(g2x2+h2x2+i2x2))/4.;
};

/* Sin[g2*t]Cos[2*t]Cos[i2*t]*/
static double evalT_17(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-g2x2-i2x2)+SinT2Pi(2+g2x2-i2x2)-SinT2Pi(2-g2x2+i2x2)+SinT2Pi(2+g2x2+i2x2))/4.;
};

/* Sin[2*t]Sin[g2*t]Cos[i2*t]*/
static double evalT_18(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-g2x2-i2x2)-CosT2Pi(2+g2x2-i2x2)+CosT2Pi(2-g2x2+i2x2)-CosT2Pi(2+g2x2+i2x2))/4.;
};

/* Sin[g2*t]Cos[i2*t]*/
static double evalT_19(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(g2x2-i2x2)+SinT2Pi(g2x2+i2x2))/2.;
};

/* Sin[h2*t]Sin[i2*t]Cos[2*t]*/
static double evalT_20(const int i2x2, const int g2x2, const int h2x2){
return (-CosT2Pi(2-h2x2-i2x2)+CosT2Pi(2+h2x2-i2x2)+CosT2Pi(2-h2x2+i2x2)-CosT2Pi(2+h2x2+i2x2))/4.;
};

/* Sin[2*t]Sin[i2*t]Cos[h2*t]*/
static double evalT_21(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-h2x2-i2x2)+CosT2Pi(2+h2x2-i2x2)-CosT2Pi(2-h2x2+i2x2)-CosT2Pi(2+h2x2+i2x2))/4.;
};

/* Cos[2*t]Cos[h2*t]Cos[i2*t]*/
static double evalT_22(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-h2x2-i2x2)+CosT2Pi(2+h2x2-i2x2)+CosT2Pi(2-h2x2+i2x2)+CosT2Pi(2+h2x2+i2x2))/4.;
};

/* Sin[2*t]Sin[h2*t]Sin[i2*t]*/
static double evalT_23(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-h2x2-i2x2)+SinT2Pi(2+h2x2-i2x2)+SinT2Pi(2-h2x2+i2x2)-SinT2Pi(2+h2x2+i2x2))/4.;
};

/* Sin[i2*t]Cos[2*t]Cos[h2*t]*/
static double evalT_24(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-h2x2-i2x2)-SinT2Pi(2+h2x2-i2x2)+SinT2Pi(2-h2x2+i2x2)+SinT2Pi(2+h2x2+i2x2))/4.;
};

/* Sin[h2*t]Cos[2*t]Cos[i2*t]*/
static double evalT_25(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-h2x2-i2x2)+SinT2Pi(2+h2x2-i2x2)-SinT2Pi(2-h2x2+i2x2)+SinT2Pi(2+h2x2+i2x2))/4.;
};

/* Sin[2*t]Sin[i2*t]Cos[2*t]*/
static double evalT_26(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(4-i2x2)-CosT2Pi(4+i2x2))/4.;
};

/* Cos[2*t]Cos[2*t]Cos[i2*t]*/
static double evalT_27(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(4-i2x2)+2*CosT2Pi(i2x2)+CosT2Pi(4+i2x2))/4.;
};

/* Sin[2*t]Sin[2*t]Sin[i2*t]*/
static double evalT_28(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(4-i2x2)+2*SinT2Pi(i2x2)-SinT2Pi(4+i2x2))/4.;
};

/* Sin[i2*t]Cos[2*t]Cos[2*t]*/
static double evalT_29(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(4-i2x2)+2*SinT2Pi(i2x2)+SinT2Pi(4+i2x2))/4.;
};

/* Sin[2*t]Cos[2*t]Cos[i2*t]*/
static double evalT_30(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(4-i2x2)+SinT2Pi(4+i2x2))/4.;
};

/* Sin[2*t]Sin[i2*t]*/
static double evalT_31(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-i2x2)-CosT2Pi(2+i2x2))/2.;
};

/* Cos[2*t]Cos[i2*t]*/
static double evalT_32(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-i2x2)+CosT2Pi(2+i2x2))/2.;
};

/* Sin[i2*t]Cos[2*t]*/
static double evalT_33(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-i2x2)+SinT2Pi(2+i2x2))/2.;
};

/* Sin[2*t]Cos[h2*t]Cos[i2*t]*/
static double evalT_34(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(2-h2x2-i2x2)+SinT2Pi(2+h2x2-i2x2)+SinT2Pi(2-h2x2+i2x2)+SinT2Pi(2+h2x2+i2x2))/4.;
};

/* Sin[2*t]Sin[h2*t]Cos[i2*t]*/
static double evalT_35(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-h2x2-i2x2)-CosT2Pi(2+h2x2-i2x2)+CosT2Pi(2-h2x2+i2x2)-CosT2Pi(2+h2x2+i2x2))/4.;
};

/* Sin[2*t]Sin[2*t]Cos[i2*t]*/
static double evalT_36(const int i2x2, const int g2x2, const int h2x2){
return (-CosT2Pi(4-i2x2)+2*CosT2Pi(i2x2)-CosT2Pi(4+i2x2))/4.;
};

/* Sin[2*t]Cos[i2*t]*/
static double evalT_37(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(2-i2x2)+SinT2Pi(2+i2x2))/2.;
};

/* Sin[h2*t]Sin[i2*t]*/
static double evalT_38(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(h2x2-i2x2)-CosT2Pi(h2x2+i2x2))/2.;
};

/* Cos[h2*t]Cos[i2*t]*/
static double evalT_39(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(h2x2-i2x2)+CosT2Pi(h2x2+i2x2))/2.;
};

/* Sin[i2*t]Cos[h2*t]*/
static double evalT_40(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(h2x2-i2x2)+SinT2Pi(h2x2+i2x2))/2.;
};

/* Sin[h2*t]Cos[i2*t]*/
static double evalT_41(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(h2x2-i2x2)+SinT2Pi(h2x2+i2x2))/2.;
};

/* Cos[i2*t]*/
static double evalT_42(const int i2x2, const int g2x2, const int h2x2){
return CosT2Pi(i2x2);
};

/* Sin[i2*t]*/
static double evalT_43(const int i2x2, const int g2x2, const int h2x2){
return SinT2Pi(i2x2);
};

/* 1*/
static double evalT_44(const int i2x2, const int g2x2, const int h2x2){
return 2*Pi;
};

/* Sin[2*t]Sin[h2*t]Cos[g2*t]*/
static double evalT_45(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-g2x2-h2x2)+CosT2Pi(2+g2x2-h2x2)-CosT2Pi(2-g2x2+h2x2)-CosT2Pi(2+g2x2+h2x2))/4.;
};

/* Sin[2*t]Sin[g2*t]Cos[h2*t]*/
static double evalT_46(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-g2x2-h2x2)-CosT2Pi(2+g2x2-h2x2)+CosT2Pi(2-g2x2+h2x2)-CosT2Pi(2+g2x2+h2x2))/4.;
};

/* Cos[2*t]Cos[g2*t]Cos[h2*t]*/
static double evalT_47(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-g2x2-h2x2)+CosT2Pi(2+g2x2-h2x2)+CosT2Pi(2-g2x2+h2x2)+CosT2Pi(2+g2x2+h2x2))/4.;
};

/* Sin[2*t]Sin[g2*t]Sin[h2*t]*/
static double evalT_48(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-g2x2-h2x2)+SinT2Pi(2+g2x2-h2x2)+SinT2Pi(2-g2x2+h2x2)-SinT2Pi(2+g2x2+h2x2))/4.;
};

/* Sin[h2*t]Cos[2*t]Cos[g2*t]*/
static double evalT_49(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-g2x2-h2x2)-SinT2Pi(2+g2x2-h2x2)+SinT2Pi(2-g2x2+h2x2)+SinT2Pi(2+g2x2+h2x2))/4.;
};

/* Sin[2*t]Cos[g2*t]Cos[h2*t]*/
static double evalT_50(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(2-g2x2-h2x2)+SinT2Pi(2+g2x2-h2x2)+SinT2Pi(2-g2x2+h2x2)+SinT2Pi(2+g2x2+h2x2))/4.;
};

/* Sin[2*t]Sin[g2*t]Cos[2*t]*/
static double evalT_51(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(4-g2x2)-CosT2Pi(4+g2x2))/4.;
};

/* Sin[2*t]Sin[2*t]Cos[g2*t]*/
static double evalT_52(const int i2x2, const int g2x2, const int h2x2){
return (-CosT2Pi(4-g2x2)+2*CosT2Pi(g2x2)-CosT2Pi(4+g2x2))/4.;
};

/* Cos[2*t]Cos[2*t]Cos[g2*t]*/
static double evalT_53(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(4-g2x2)+2*CosT2Pi(g2x2)+CosT2Pi(4+g2x2))/4.;
};

/* Sin[2*t]Sin[2*t]Sin[g2*t]*/
static double evalT_54(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(4-g2x2)+2*SinT2Pi(g2x2)-SinT2Pi(4+g2x2))/4.;
};

/* Sin[2*t]Cos[2*t]Cos[g2*t]*/
static double evalT_55(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(4-g2x2)+SinT2Pi(4+g2x2))/4.;
};

/* Sin[2*t]Sin[g2*t]*/
static double evalT_56(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-g2x2)-CosT2Pi(2+g2x2))/2.;
};

/* Cos[2*t]Cos[g2*t]*/
static double evalT_57(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-g2x2)+CosT2Pi(2+g2x2))/2.;
};

/* Sin[2*t]Cos[g2*t]*/
static double evalT_58(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(2-g2x2)+SinT2Pi(2+g2x2))/2.;
};

/* Sin[g2*t]Cos[2*t]Cos[h2*t]*/
static double evalT_59(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-g2x2-h2x2)+SinT2Pi(2+g2x2-h2x2)-SinT2Pi(2-g2x2+h2x2)+SinT2Pi(2+g2x2+h2x2))/4.;
};

/* Sin[g2*t]Sin[h2*t]Cos[2*t]*/
static double evalT_60(const int i2x2, const int g2x2, const int h2x2){
return (-CosT2Pi(2-g2x2-h2x2)+CosT2Pi(2+g2x2-h2x2)+CosT2Pi(2-g2x2+h2x2)-CosT2Pi(2+g2x2+h2x2))/4.;
};

/* Sin[g2*t]Cos[2*t]Cos[2*t]*/
static double evalT_61(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(4-g2x2)+2*SinT2Pi(g2x2)+SinT2Pi(4+g2x2))/4.;
};

/* Sin[g2*t]Cos[2*t]*/
static double evalT_62(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-g2x2)+SinT2Pi(2+g2x2))/2.;
};

/* Sin[2*t]Sin[h2*t]Cos[2*t]*/
static double evalT_63(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(4-h2x2)-CosT2Pi(4+h2x2))/4.;
};

/* Sin[2*t]Sin[2*t]Cos[h2*t]*/
static double evalT_64(const int i2x2, const int g2x2, const int h2x2){
return (-CosT2Pi(4-h2x2)+2*CosT2Pi(h2x2)-CosT2Pi(4+h2x2))/4.;
};

/* Cos[2*t]Cos[2*t]Cos[h2*t]*/
static double evalT_65(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(4-h2x2)+2*CosT2Pi(h2x2)+CosT2Pi(4+h2x2))/4.;
};

/* Sin[2*t]Sin[2*t]Sin[h2*t]*/
static double evalT_66(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(4-h2x2)+2*SinT2Pi(h2x2)-SinT2Pi(4+h2x2))/4.;
};

/* Sin[h2*t]Cos[2*t]Cos[2*t]*/
static double evalT_67(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(4-h2x2)+2*SinT2Pi(h2x2)+SinT2Pi(4+h2x2))/4.;
};

/* Sin[2*t]Cos[2*t]Cos[h2*t]*/
static double evalT_68(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(4-h2x2)+SinT2Pi(4+h2x2))/4.;
};

/* Sin[2*t]Sin[2*t]Cos[2*t]*/
static double evalT_69(const int i2x2, const int g2x2, const int h2x2){
return 0;
};

/* Cos[2*t]Cos[2*t]Cos[2*t]*/
static double evalT_70(const int i2x2, const int g2x2, const int h2x2){
return 0;
};

/* Sin[2*t]Sin[2*t]Sin[2*t]*/
static double evalT_71(const int i2x2, const int g2x2, const int h2x2){
return 0;
};

/* Sin[2*t]Cos[2*t]Cos[2*t]*/
static double evalT_72(const int i2x2, const int g2x2, const int h2x2){
return 0;
};

/* Sin[2*t]Sin[2*t]*/
static double evalT_73(const int i2x2, const int g2x2, const int h2x2){
return Pi;
};

/* Cos[2*t]Cos[2*t]*/
static double evalT_74(const int i2x2, const int g2x2, const int h2x2){
return Pi;
};

/* Sin[2*t]Cos[2*t]*/
static double evalT_75(const int i2x2, const int g2x2, const int h2x2){
return 0;
};

/* Sin[2*t]Sin[h2*t]*/
static double evalT_76(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-h2x2)-CosT2Pi(2+h2x2))/2.;
};

/* Cos[2*t]Cos[h2*t]*/
static double evalT_77(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(2-h2x2)+CosT2Pi(2+h2x2))/2.;
};

/* Sin[h2*t]Cos[2*t]*/
static double evalT_78(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(2-h2x2)+SinT2Pi(2+h2x2))/2.;
};

/* Sin[2*t]Cos[h2*t]*/
static double evalT_79(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(2-h2x2)+SinT2Pi(2+h2x2))/2.;
};

/* Cos[2*t]*/
static double evalT_80(const int i2x2, const int g2x2, const int h2x2){
return 0;
};

/* Sin[2*t]*/
static double evalT_81(const int i2x2, const int g2x2, const int h2x2){
return 0;
};

/* Cos[g2*t]Cos[h2*t]*/
static double evalT_82(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(g2x2-h2x2)+CosT2Pi(g2x2+h2x2))/2.;
};

/* Sin[h2*t]Cos[g2*t]*/
static double evalT_83(const int i2x2, const int g2x2, const int h2x2){
return (-SinT2Pi(g2x2-h2x2)+SinT2Pi(g2x2+h2x2))/2.;
};

/* Cos[g2*t]*/
static double evalT_84(const int i2x2, const int g2x2, const int h2x2){
return CosT2Pi(g2x2);
};

/* Sin[g2*t]Cos[h2*t]*/
static double evalT_85(const int i2x2, const int g2x2, const int h2x2){
return (SinT2Pi(g2x2-h2x2)+SinT2Pi(g2x2+h2x2))/2.;
};

/* Sin[g2*t]Sin[h2*t]*/
static double evalT_86(const int i2x2, const int g2x2, const int h2x2){
return (CosT2Pi(g2x2-h2x2)-CosT2Pi(g2x2+h2x2))/2.;
};

/* Sin[g2*t]*/
static double evalT_87(const int i2x2, const int g2x2, const int h2x2){
return SinT2Pi(g2x2);
};

/* Cos[h2*t]*/
static double evalT_88(const int i2x2, const int g2x2, const int h2x2){
return CosT2Pi(h2x2);
};

/* Sin[h2*t]*/
static double evalT_89(const int i2x2, const int g2x2, const int h2x2){
return SinT2Pi(h2x2);
};

};

#endif // CYLSIN_EVAL_T_H